let fs = require("fs");
let path = require("path");
let moment = require("moment");

let logPath;
let QH_NODE_ENV = process.env.QH_NODE_ENV || "prod";
let QH_VERBOSE_LOG = process.env.QH_VERBOSE_LOG || "false";
let QH_WARN_LOG = process.env.QH_WARN_LOG || "true";
let QH_CONSOLE = process.env.QH_CONSOLE || "false";
let QH_SERVER_NAME = process.env.QH_SERVER_NAME;
let PORT = process.env.PORT || "0";
let Debug = {
  CONSOLE: !(QH_CONSOLE === "false"),
  VERBOSE: QH_VERBOSE_LOG === "true",
  WARN: !(QH_WARN_LOG === "false"),
  DEBUG: true, // 这是软件运行模式, 在代码中设置
  TEST: false, // 这是软件运行模式, 在代码中设置
  DEV: QH_NODE_ENV.toUpperCase().indexOf("DEV") > -1,
  UI: QH_NODE_ENV.toUpperCase().indexOf("UI") > -1
};
if (QH_NODE_ENV.toUpperCase().indexOf("DEV") > -1) {
  Debug.ENV = "dev";
} else if (QH_NODE_ENV === "test") {
  Debug.ENV = "test";
} else {
  Debug.ENV = "prod";
}
Debug.SERVER_NAME = QH_SERVER_NAME ? `${Debug.ENV}.${QH_SERVER_NAME}` : Debug.ENV;

Debug.setLogPath = (side = "client") => {
  logPath = {
    VERBOSE: path.join(
      __dirname,
      "../../log/",
      moment().format("YYYY-MM-DD-") + side + PORT + "-verbose.log"
    ),
    INFO: path.join(
      __dirname,
      "../../log/",
      moment().format("YYYY-MM-DD-") + side + PORT + "-info.log"
    ),
    ERROR: path.join(
      __dirname,
      "../../log/",
      moment().format("YYYY-MM-DD-") + side + PORT + "-error.log"
    ),
    DEBUG: path.join(
      __dirname,
      "../../log/",
      moment().format("YYYY-MM-DD-") + side + PORT + "-debug.log"
    )
  };
  logPath.WARN = logPath.DEBUG;
};
Debug.setLogPath();
// console.warn("ENV:", { QH_NODE_ENV, QH_VERBOSE_LOG, QH_WARN_LOG, QH_CONSOLE, Debug, logPath });
if (!fs.existsSync(path.join(__dirname, "../../log"))) {
  try {
    fs.mkdirSync(path.join(__dirname, "../../log"));
  } catch (e) {
    console.error(e);
  }
}

Debug.d = Debug.debug = function() {
  if (Debug.CONSOLE) {
    console.log.apply(null, arguments);
  }
  if (Debug.VERBOSE) {
    Debug.log("VERBOSE", JSON.stringify(Array.prototype.slice.call(arguments), null, "  "));
  }
};

Debug.v = Debug.verbose = function() {
  if (Debug.VERBOSE) {
    Debug.log("VERBOSE", JSON.stringify(Array.prototype.slice.call(arguments), null, "  "));
  }
};

// test模式专用日志
Debug.i = Debug.info = function Info() {
  if (Debug.TEST) {
    console.info.apply(null, arguments);
  }
};

Debug.e = Debug.error = function Error() {
  console.error.apply(null, arguments);
  if (Debug.TEST) {
    return;
  }
  let arr = Array.prototype.slice.call(arguments);
  arr = arr.map(e => {
    if (e && e.stack) {
      return e.stack.split("\n");
    }
    return e;
  });
  Debug.log("ERROR", JSON.stringify(arr, null, "  "));
};

Debug.w = Debug.warn = function Warn() {
  console.warn.apply(null, arguments);
  if (Debug.WARN) {
    Debug.log("WARN", JSON.stringify(Array.prototype.slice.call(arguments), null, "  "));
  }
};

Debug.setConsole = c => (Debug.CONSOLE = c);
Debug.setVerbose = v => (Debug.VERBOSE = v);
Debug.setTest = t => {
  Debug.TEST = t;
  // Debug.CONSOLE = false;
};
Debug.setDebug = d => (Debug.DEBUG = d);
Debug.setUI = ui => (Debug.UI = ui);

// 写入文件
Debug.log = function(type, ...arg) {
  try {
    fs.appendFile(
      logPath[type.toUpperCase()],
      `[${moment().format("YYYY-MM-DD HH:mm:ss.SSS")}] ${arg
        .map(o => JSON.stringify(o))
        .join(", ")}\n`,
      () => {}
    );
  } catch (e) {}
};

let timeLogs = [];
Debug.t = Debug.time = function(content, opt = {}) {
  if (!Debug.VERBOSE) {
    return;
  }
  if (opt.force) {
    console.log("time logs:", timeLogs);
    timeLogs = [];
  } else {
    if (!timeLogs.length) {
      timeLogs.push({
        timer: process.hrtime(),
        content
      });
    } else {
      timeLogs.push({
        timer: process.hrtime(timeLogs[0].timer),
        content
      });
    }
  }
};

module.exports = Debug;
